import json
from math import pi, cos


# get_markers requests all marker data or valid markers, converts the data to json, and writes
# the data to the response object
def get_all_markers(cursor, valid: bool, waterBodyName):
    try:
        sql_query = '''
            SELECT m.MeasurementID, m.SensorID, m.TimeMeasured, m.CenterLat, m.CenterLon,
                   s.SensorType, s.Active, 
                   b.Name,
                   d.SubDivisionID, d.GroupID, d.MinimumThickness, 
                   d.AverageThickness, d.CenterLatitude, d.CenterLongitude, 
                   d.Accuracy
            FROM Measurement m
            INNER JOIN Sensor s ON m.SensorID = s.SensorID
            INNER JOIN BodyOfWater b ON m.WaterBodyName = b.Name
            LEFT JOIN SubDivision d ON m.MeasurementID = d.MeasurementID
            WHERE b.Name = 'Mjosa'
        '''

        if valid:  # Append time restriction to WHERE statement, NB temporarily hardcoded to 5 days
            sql_query += ' AND m.TimeMeasured > DATE_SUB(NOW(), INTERVAL 5 DAY)'

        cursor.execute(sql_query)

        rows = cursor.fetchall()

        # Container for all fetched measurement objects
        measurement_data = {}

        # Iterate over all fetched rows
        for row in rows:
            measurement_id = row[0]

            # Create subdivision new object
            sub_division = {
                'SubdivID': row[8],
                'GroupID': row[9],
                'MinThickness': row[10],
                'AvgThickness': row[11],
                'CenLatitude': row[12],
                'CenLongitude': row[13],
                'Accuracy': row[14]
            }

            # Check if measurement ID already exists in measurement_data
            if measurement_id in measurement_data:
                # Create new subdivision within measurement if it does not already exist
                if sub_division not in measurement_data[measurement_id]['Subdivisions']:
                    measurement_data[measurement_id]['Subdivisions'].append(sub_division)

            else:
                # Create a new entry for measurement_id if it does not already exist in the list
                measurement_data[measurement_id] = {
                    'MeasurementID': measurement_id,
                    'TimeMeasured': row[2],
                    'CenterLat': row[3],
                    'CenterLon': row[4],
                    'Sensor': {  # Each measurement only has one related sensor
                        'SensorID': row[1],
                        'SensorType': row[5],
                        'Active': bool(row[6])
                    },
                    'Subdivisions': [sub_division],  # Array of sub_division objects
                    'Corners': calculate_corners(row[3], row[4]),
                    # Returns list of corners calculated based on center coordinates
                }

        # Convert dictionary values to list of measurements
        data = list(measurement_data.values())

        if len(rows) == 0 or len(data) == 0:  # Return 500 and empty list if no data is found
            print(f"Error in querying database")
            marker_data = '[]'
        else:
            # Convert list of dictionaries to JSON
            marker_data = json.dumps(data, indent=4)

    except Exception as e:
        print(f"Error in querying database: {e}")
        marker_data = '[]'

    return marker_data


EARTH = 6378.137  # Radius of the earth in kilometer
METER = (1 / ((2 * pi / 360) * EARTH)) / 1000  # 1 meter in degree
OFFSET = 20  # Offset in meters


def calculate_corners(lat, lng):
    """Calculate corners of polygon based on a center coordinate

    Arguments:
    lat -- center latitude
    lng -- center longitude
    """
    # From https://stackoverflow.com/questions/7477003/calculating-new-longitude-latitude-from-old-n-meters
    # Formulas:
    '''
    lat_pos = lat + (OFFSET * METER)
    lng_pos = lng + (OFFSET * METER) / cos(lat * (pi / 180))

    lat_neg = lat - (OFFSET * METER)
    lng_neg = lng - (OFFSET * METER) / cos(lat * (pi / 180))

    return [
        (lat_neg, lng_pos),
        (lat_pos, lng_pos),
        (lat_pos, lng_neg),
        (lat_neg, lng_neg)
    ]    '''

    return [(60.7798, 10.7062),  # NB: temporary hardcoded values
            (60.7553, 10.7433),
            (60.7718, 10.7975),
            (60.7966, 10.7405)]
